@@ -22,7 +22,7 @@ from ._response import BatchResponse, Response
 from ._serializer import JSONSerializer, PickleSerializer
 from ._wsgi import WSGIHandler
 
-__all__: Final[list[str]] = [
+__all__: Final[tuple[str, ...]] = (
     "BatchRequest",
     "BatchResponse",
     "Dispatcher",
@@ -33,6 +33,6 @@ __all__: Final[list[str]] = [
     "Request",
     "Response",
     "WSGIHandler"
-]
+)
 
-__version__: Final[str] = "2.2.3"
+__version__: Final[str] = "2.4.0"
