@@ -14,18 +14,18 @@
 # limitations under the License.
 
 from abc import ABCMeta, abstractmethod
-from collections.abc import Iterable, Iterator, MutableMapping, MutableSequence
+from collections import UserList as List
+from collections.abc import Iterable, MutableMapping, MutableSequence
 from numbers import Number
-from typing import Any, Final, TypeAlias, TypeVar
+from typing import Any, Final, Literal, TypeAlias, TypeVar, Union
 
-from ._errors import Error, ErrorEnum
-from ._utilities import Undefined, UndefinedType, make_hashable, partition
+from ._errors import BaseError, Error, ErrorEnum
+from ._utilities import Undefined, UndefinedType, make_hashable
 
-__all__: Final[list[str]] = ["BatchRequest", "Request"]
+__all__: Final[tuple[str, ...]] = ("BatchRequest", "Request")
 
-_T = TypeVar("_T")
-_Args: TypeAlias = MutableSequence[Any]
-_Kwargs: TypeAlias = MutableMapping[str, Any]
+_Params: TypeAlias = list[object] | dict[str, object]
+_BaseRequestOrError = TypeVar("_BaseRequestOrError", bound=Union["BaseRequest", BaseError])
 
 # Method names that begins with the "rpc." prefix are reserved for internal usage only:
 _INVALID_SIGNATURE: Final[str] = "\x72\x70\x63\x2E"
@@ -51,7 +51,7 @@ class BaseRequest(metaclass=ABCMeta):
 
     @property
     @abstractmethod
-    def request_id(self) -> str | Number | UndefinedType:
+    def request_id(self) -> str | float | UndefinedType:
         raise NotImplementedError
 
     @property
@@ -59,22 +59,24 @@ class BaseRequest(metaclass=ABCMeta):
     def is_notification(self) -> bool:
         raise NotImplementedError
 
-    @classmethod
+    @staticmethod
     @abstractmethod
-    def from_json(cls: type[_T], obj: dict[str, Any], /) -> _T | Error:
+    def from_json(obj: dict[str, Any]) -> Union["BaseRequest", BaseError]:
         raise NotImplementedError
 
 
-class BaseBatchRequest(Iterable[BaseRequest | Error]):
+class BaseBatchRequest(List[_BaseRequestOrError], metaclass=ABCMeta):
     __slots__: tuple[str, ...] = ()
 
-    @abstractmethod
-    def __len__(self) -> int:
-        raise NotImplementedError
+    def __hash__(self) -> int:
+        return hash(tuple(self.data))
 
-    @classmethod
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__qualname__}({self.data!r})>"
+
+    @staticmethod
     @abstractmethod
-    def from_json(cls: type[_T], obj: list[dict[str, Any]], /) -> _T:
+    def from_json(iterable: Iterable[dict[str, Any]]) -> "BaseBatchRequest[_BaseRequestOrError]":
         raise NotImplementedError
 
 
@@ -82,65 +84,61 @@ class Request(BaseRequest):
     """
     Base JSON-RPC request object.
 
-    :param method: The :py:class:`str` object containing the name of the method to be invoked by the :class:`jsonrpc.WSGIHandler` instance.
-    :param params: The object of type :py:class:`collections.abc.MutableSequence` or :py:class:`collections.abc.MutableMapping` that holds
-        the parameter values to be used during the invocation of the method. May be omitted if provided method has no parameters for
-        example.
+    :param method: The :py:class:`str` object containing the name of the method.
+    :param params: The object of type :py:class:`list` or :py:class:`dict` that holds the parameter values
+        to be used during the invocation of the method.
+        May be omitted if provided method has no parameters for example.
     :param request_id: The :py:class:`str` object or any type of :py:class:`numbers.Number` object which represents an identifier
         of the request instance. May be omitted. If its value omitted, the request assumed to be a notification.
     :raises jsonrpc.Error: If the request method isn't a string or have a \"rpc.\" prefix, if the request parameters
         aren't an objects of type :py:class:`collections.abc.MutableSequence` or :py:class:`collections.abc.MutableMapping` if provided,
         also if the request identifier isn't an object of type :py:class:`str` or :py:class:`numbers.Number` if provided.
     """
-    __slots__: list[str] = ["_method", "_params", "_id"]
+    __slots__: tuple[str, ...] = ("_method", "_params", "_id")
 
     def __init__(self, *,
         method: str,
-        params: _Args | _Kwargs | UndefinedType = Undefined,
-        request_id: str | Number | UndefinedType = Undefined
+        params: _Params | UndefinedType = Undefined,
+        request_id: str | float | UndefinedType = Undefined
     ) -> None:
-        self._method: Final[str] = self._is_method_valid(method=method)
-        self._params: Final[_Args | _Kwargs | UndefinedType] = self._is_params_valid(params=params)
-        self._id: Final[str | Number | UndefinedType] = self._is_request_id_valid(request_id=request_id)
+        self._method: Final[str] = self._validate_method(method) and method
+        self._params: Final[_Params | UndefinedType] = self._validate_params(params) and params
+        self._id: Final[str | float | UndefinedType] = self._validate_id(request_id) and request_id
 
     def __repr__(self) -> str:
-        return f"<{__package__}.{self.__class__.__qualname__}(" \
+        return f"<{self.__class__.__qualname__}(" \
             f"method={self._method!r}, " \
             f"params={self._params!r}, " \
             f"request_id={self._id!r})>"
 
     def __hash__(self) -> int:
-        return hash(self._method) ^ hash(make_hashable(self._params)) ^ hash(self._id)
+        return hash((self._method, make_hashable(self._params), self._id))
 
-    def __eq__(self, obj: Any, /) -> bool:
-        return isinstance(obj, self.__class__) \
-            and type(self._method) is type(obj._method) and self._method == obj._method \
-            and type(self._params) is type(obj._params) and self._params == obj._params \
-            and type(self._id) is type(obj._id) and self._id == obj._id
+    def __eq__(self, obj: Any) -> bool:
+        if not isinstance(obj, self.__class__):
+            return NotImplemented
+        return (self._method, self._params, self._id) == (obj.method, obj._params, obj._id)
 
-    def _is_method_valid(self, *, method: _T) -> _T:
+    def _validate_method(self, method: Any) -> Literal[True]:
         if not isinstance(method, str) or method.startswith(_INVALID_SIGNATURE, 0, len(_INVALID_SIGNATURE)):
             raise Error(
                 code=ErrorEnum.INVALID_REQUEST,
                 message=f"Request method must be a string and doesn't have a {_INVALID_SIGNATURE!r} prefix")
+        return True
 
-        return method
-
-    def _is_params_valid(self, *, params: _T) -> _T:
+    def _validate_params(self, params: Any) -> Literal[True]:
         if not isinstance(params, MutableSequence | MutableMapping | UndefinedType):
             raise Error(
                 code=ErrorEnum.INVALID_REQUEST,
                 message=f"Request params must be a sequence or mapping, not a {type(params).__name__!r}")
+        return True
 
-        return params
-
-    def _is_request_id_valid(self, *, request_id: _T) -> _T:
+    def _validate_id(self, request_id: Any) -> Literal[True]:
         if not isinstance(request_id, str | Number | UndefinedType):
             raise Error(
                 code=ErrorEnum.INVALID_REQUEST,
                 message=f"Request id must be an optional string or number, not a {type(request_id).__name__!r}")
-
-        return request_id
+        return True
 
     @property
     def method(self) -> str:
@@ -150,21 +148,21 @@ class Request(BaseRequest):
         return self._method
 
     @property
-    def args(self) -> tuple[Any, ...]:
+    def args(self) -> tuple[object, ...]:
         """
         Returns the :py:class:`tuple` object containing positional arguments of the method.
         """
-        return tuple(self._params) if isinstance(self._params, MutableSequence) else ()
+        return tuple(params) if isinstance(params := self._params, MutableSequence) else ()
 
     @property
-    def kwargs(self) -> dict[str, Any]:
+    def kwargs(self) -> dict[str, object]:
         """
         Returns the :py:class:`dict` object containing keyword arguments of the method.
         """
-        return dict(self._params) if isinstance(self._params, MutableMapping) else {}
+        return params if isinstance(params := self._params, MutableMapping) else {}
 
     @property
-    def request_id(self) -> str | Number | UndefinedType:
+    def request_id(self) -> str | float | UndefinedType:
         """
         Returns the :py:class:`str` object or any type of :py:class:`numbers.Number` object
         containing the identifier of the request if its value is set.
@@ -178,20 +176,19 @@ class Request(BaseRequest):
         """
         return isinstance(self._id, UndefinedType)
 
-    @classmethod
-    def from_json(cls: type[_T], obj: dict[str, Any], /) -> _T | Error:
+    @staticmethod
+    def from_json(obj: dict[str, Any]) -> Union["Request", Error]:
         """
-        The class method for creating the :class:`jsonrpc.Request` object from :py:class:`dict` object.
-        Primarily used by the :class:`jsonrpc.WSGIHandler` instance.
+        The static method for creating the :class:`jsonrpc.Request` object from :py:class:`dict` object.
         Unlike the :class:`jsonrpc.Request` constructor, doesn't raises any exceptions by validations,
         it returns the :class:`jsonrpc.Error` as is.
 
         Example usage::
 
             >>> Request.from_json({"jsonrpc": "2.0", "method": "foobar", "id": 1})
-            <jsonrpc.Request(method="foobar", params=Undefined, request_id=1)>
+            <Request(method="foobar", params=Undefined, request_id=1)>
             >>> Request.from_json({"not_jsonrpc": True})
-            <jsonrpc.Error(code=-32600, message="Invalid request object", data={"not_jsonrpc": True})>
+            <Error(code=-32600, message="Invalid request object", data={"not_jsonrpc": True})>
         """
         try:
             match obj:
@@ -211,37 +208,16 @@ class Request(BaseRequest):
 
 class BatchRequest(BaseBatchRequest[Request | Error]):
     """
-    The :py:class:`collections.abc.Iterable` subclass representing the collection
+    The :py:class:`collections.UserList` subclass representing the collection
     of :class:`jsonrpc.Request` and :class:`jsonrpc.Error` objects.
     """
-    __slots__: list[str] = ["_requests"]
+    __slots__: tuple[str, ...] = ()
 
-    def __init__(self, requests: Iterable[Request | Error], /) -> None:
-        self._requests: Final[tuple[Request | Error, ...]] = tuple(requests)
-
-    def __repr__(self) -> str:
-        return f"<{__package__}.{self.__class__.__qualname__}(\u2026)>"
-
-    def __hash__(self) -> int:
-        return hash(frozenset(self._requests)) ^ len(self._requests)
-
-    def __eq__(self, obj: Any, /) -> bool:
-        return isinstance(obj, self.__class__) \
-            and len(self._requests) == len(obj._requests) \
-            and frozenset(self._requests) == frozenset(obj._requests)
-
-    def __iter__(self) -> Iterator[Request | Error]:
-        return partition(lambda obj: isinstance(obj, Request) and obj.is_notification, self._requests)
-
-    def __len__(self) -> int:
-        return len(self._requests)
-
-    @classmethod
-    def from_json(cls: type[_T], obj: list[dict[str, Any]], /) -> _T:
+    @staticmethod
+    def from_json(iterable: Iterable[dict[str, Any]]) -> "BatchRequest":
         """
-        The class method for creating the :class:`jsonrpc.BatchRequest` object from :py:class:`collections.abc.Iterable`
+        The static method for creating the :class:`jsonrpc.BatchRequest` object from :py:class:`collections.abc.Iterable`
         of :py:class:`dict` objects.
-        Primarily used by the :class:`jsonrpc.WSGIHandler` instance.
         Similar to :func:`jsonrpc.Request.from_json` function it doesn't raises any exceptions.
 
         Example usage::
@@ -250,6 +226,6 @@ class BatchRequest(BaseBatchRequest[Request | Error]):
             ...     {"jsonrpc": "2.0", "method": "foobar", "id": 1},
             ...     {"not_jsonrpc": True}
             ... ])
-            <jsonrpc.BatchRequest((<jsonrpc.Request(\u2026)>, <jsonrpc.Error(\u2026)>))>
+            <BatchRequest((<Request(\u2026)>, <Error(\u2026)>))>
         """
-        return BatchRequest(map(Request.from_json, obj))
+        return BatchRequest(map(Request.from_json, iterable))
