@@ -14,17 +14,17 @@
 # limitations under the License.
 
 from abc import ABCMeta, abstractmethod
-from collections.abc import Iterable, Iterator
+from collections import UserList as List
 from numbers import Number
 from types import NoneType
-from typing import Any, Final, TypeVar, overload
+from typing import Any, Final, Literal, TypeVar, overload
 
 from ._errors import Error
-from ._utilities import Undefined, UndefinedType, make_hashable, partition
+from ._utilities import Undefined, UndefinedType, make_hashable
 
-__all__: Final[list[str]] = ["BatchResponse", "Response"]
+__all__: Final[tuple[str, ...]] = ("BatchResponse", "Response")
 
-_T = TypeVar("_T")
+_BaseResponse = TypeVar("_BaseResponse", bound="BaseResponse")
 
 
 class BaseResponse(metaclass=ABCMeta):
@@ -42,7 +42,7 @@ class BaseResponse(metaclass=ABCMeta):
 
     @property
     @abstractmethod
-    def response_id(self) -> str | Number | UndefinedType | None:
+    def response_id(self) -> str | float | UndefinedType | None:
         raise NotImplementedError
 
     @property
@@ -56,12 +56,14 @@ class BaseResponse(metaclass=ABCMeta):
         raise NotImplementedError
 
 
-class BaseBatchResponse(Iterable[BaseResponse]):
+class BaseBatchResponse(List[_BaseResponse], metaclass=ABCMeta):
     __slots__: tuple[str, ...] = ()
 
-    @abstractmethod
-    def __len__(self) -> int:
-        raise NotImplementedError
+    def __hash__(self) -> int:
+        return hash(tuple(self.data))
+
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__qualname__}({self.data!r})>"
 
     @property
     @abstractmethod
@@ -82,54 +84,52 @@ class Response(BaseResponse):
     :raises TypeError: If both or no one ``body`` or ``error`` attributes are set
         or response identifier isn't the same type as request identifier.
     """
-    __slots__: list[str] = ["_body", "_error", "_id"]
+    __slots__: tuple[str, ...] = ("_body", "_error", "_id")
 
     @overload
     def __init__(self, *,
         body: Any,
-        response_id: str | Number | UndefinedType | None = ...
+        response_id: str | float | UndefinedType | None = ...
     ) -> None: ...
 
     @overload
     def __init__(self, *,
         error: Error,
-        response_id: str | Number | UndefinedType | None = ...
+        response_id: str | float | UndefinedType | None = ...
     ) -> None: ...
 
     def __init__(self, *,
         body: Any = Undefined,
         error: Error | UndefinedType = Undefined,
-        response_id: str | Number | UndefinedType | None = Undefined
+        response_id: str | float | UndefinedType | None = Undefined
     ) -> None:
-        self._is_body_and_error_valid(body=body, error=error)
+        self._validate_body_and_error(body, error)
         self._body: Final[Any] = body
         self._error: Final[Error | UndefinedType] = error
-        self._id: Final[str | Number | UndefinedType | None] = self._is_response_id_valid(response_id=response_id)
+        self._id: Final[str | float | UndefinedType | None] = self._validate_id(response_id) and response_id
 
     def __repr__(self) -> str:
-        return f"<{__package__}.{self.__class__.__qualname__}(" \
+        return f"<{self.__class__.__qualname__}(" \
             f"body={self._body!r}, " \
             f"error={self._error!r}, " \
             f"response_id={self._id!r})>"
 
     def __hash__(self) -> int:
-        return hash(make_hashable(self._body)) ^ hash(self._error) ^ hash(self._id)
+        return hash((make_hashable(self._body), self._error, self._id))
 
-    def __eq__(self, obj: Any, /) -> bool:
-        return isinstance(obj, self.__class__) \
-            and type(self._body) is type(obj._body) and self._body == obj._body \
-            and type(self._error) is type(obj._error) and self._error == obj._error \
-            and type(self._id) is type(obj._id) and self._id == obj._id
+    def __eq__(self, obj: Any) -> bool:
+        if not isinstance(obj, self.__class__):
+            return NotImplemented
+        return (self._body, self._error, self._id) == (obj._body, obj._error, obj._id)
 
-    def _is_body_and_error_valid(self, *, body: Any, error: Error | UndefinedType) -> None:
+    def _validate_body_and_error(self, body: Any, error: Error | UndefinedType) -> None:
         if isinstance(body, UndefinedType) == isinstance(error, UndefinedType):
             raise TypeError("Either \"body\" or \"error\" attribute must be set")
 
-    def _is_response_id_valid(self, *, response_id: _T) -> _T:
+    def _validate_id(self, response_id: Any) -> Literal[True]:
         if not isinstance(response_id, str | Number | UndefinedType | NoneType):
             raise TypeError(f"Response id must be an optional string or number, not a {type(response_id).__name__!r}")
-
-        return response_id
+        return True
 
     @property
     def body(self) -> Any:
@@ -139,9 +139,8 @@ class Response(BaseResponse):
 
         :raises AttributeError: If the response is erroneous.
         """
-        if not self.is_successful:
+        if isinstance(self._body, UndefinedType):
             raise AttributeError("Erroneous response hasn't \"body\" attribute")
-
         return self._body
 
     @property
@@ -151,13 +150,12 @@ class Response(BaseResponse):
 
         :raises AttributeError: If the response is successful.
         """
-        if self.is_successful:
+        if isinstance(self._error, UndefinedType):
             raise AttributeError("Successful response hasn't \"error\" attribute")
-
         return self._error
 
     @property
-    def response_id(self) -> str | Number | UndefinedType | None:
+    def response_id(self) -> str | float | UndefinedType | None:
         """
         Returns the :py:class:`str` object or any type of :py:class:`numbers.Number` object
         representing the identifier of the response.
@@ -177,7 +175,6 @@ class Response(BaseResponse):
     def json(self) -> dict[str, Any]:
         """
         Returns the :py:class:`dict` object needed for the serialization.
-        Primarily used by the :class:`jsonrpc.WSGIHandler` instance.
 
         Example successful response::
 
@@ -188,16 +185,16 @@ class Response(BaseResponse):
         Example erroneous response::
 
             >>> error: Error = Error(code=ErrorEnum.INTERNAL_ERROR, message="Unexpected error")
-            >>> response: Response = Response(error=error, response_id="6ba7b810-9dad-11d1-80b4-00c04fd430c8")
+            >>> response: Response = Response(error=error, response_id="6ba7b810")
             >>> response.json
-            {"jsonrpc": "2.0", "error": {"code": -32603, "message": "Unexpected error"}, "id": "6ba7b810-9dad-11d1-80b4-00c04fd430c8"}
+            {"jsonrpc": "2.0", "error": {"code": -32603, "message": "Unexpected error"}, "id": "6ba7b810"}
         """
         obj: dict[str, Any] = {"jsonrpc": "2.0"}
 
-        try:
-            obj |= {"result": self.body}
-        except AttributeError:
-            obj |= {"error": self.error.json}
+        if isinstance(error := self._error, UndefinedType):
+            obj |= {"result": self._body}
+        else:
+            obj |= {"error": error.json}
         if not isinstance(response_id := self._id, UndefinedType):
             obj |= {"id": response_id}
 
@@ -206,36 +203,15 @@ class Response(BaseResponse):
 
 class BatchResponse(BaseBatchResponse[Response]):
     """
-    The :py:class:`collections.abc.Iterable` subclass representing the unordered collection
-    of :class:`jsonrpc.Response` unique objects.
+    The :py:class:`collections.UserList` subclass representing the collection
+    of :class:`jsonrpc.Response` objects.
     """
-    __slots__: list[str] = ["_responses"]
-
-    def __init__(self, responses: Iterable[Response], /) -> None:
-        self._responses: Final[frozenset[Response]] = frozenset(responses)
-
-    def __repr__(self) -> str:
-        return f"<{__package__}.{self.__class__.__qualname__}(\u2026)>"
-
-    def __hash__(self) -> int:
-        return hash(self._responses) ^ len(self._responses)
-
-    def __eq__(self, obj: Any, /) -> bool:
-        return isinstance(obj, self.__class__) \
-            and len(self._responses) == len(obj._responses) \
-            and self._responses == obj._responses
-
-    def __iter__(self) -> Iterator[Response]:
-        return partition(lambda response: response.is_successful, self._responses)
-
-    def __len__(self) -> int:
-        return len(self._responses)
+    __slots__: tuple[str, ...] = ()
 
     @property
     def json(self) -> list[dict[str, Any]]:
         """
         Returns the :py:class:`list` of :py:class:`dict` objects needed for the serialization.
-        Primarily used by the :class:`jsonrpc.WSGIHandler` instance.
 
         Example output::
 
@@ -244,12 +220,12 @@ class BatchResponse(BaseBatchResponse[Response]):
             ...     Response(error=Error(
             ...         code=ErrorEnum.INTERNAL_ERROR,
             ...         message="Unexpected error"
-            ...     ), response_id="6ba7b810-9dad-11d1-80b4-00c04fd430c8")
+            ...     ), response_id="6ba7b810")
             ... ])
             >>> response.json
             [
                 {"jsonrpc": "2.0", "result": "foobar", "id": 1024},
-                {"jsonrpc": "2.0", "error": {"code": -32603, "message": "Unexpected error"}, "id": "6ba7b810-9dad-11d1-80b4-00c04fd430c8"}
+                {"jsonrpc": "2.0", "error": {"code": -32603, "message": "Unexpected error"}, "id": "6ba7b810"}
             ]
         """
         return list(map(lambda response: response.json, self))
