@@ -15,35 +15,26 @@
 
 from collections.abc import Iterable, Iterator, MutableMapping, MutableSequence
 from types import TracebackType
-from typing import Any, Final, Protocol, TypeAlias, TypeVar
+from typing import Any, Final, Protocol, TypeAlias, Union
 
-__all__: Final[list[str]] = [
+__all__: Final[tuple[str, ...]] = (
     "Headers",
     "InputStream",
     "OptExcInfo",
     "StartResponse",
-    "SupportsKeysAndGetItem",
     "WSGIEnvironment"
-]
-
-_K = TypeVar("_K")
-_V = TypeVar("_V")
+)
 
 Headers: TypeAlias = MutableSequence[tuple[str, str]]
-OptExcInfo: TypeAlias = tuple[type[Exception], Exception, TracebackType] | tuple[None, None, None]
+OptExcInfo: TypeAlias = Union[tuple[type[Exception], Exception, TracebackType | None], tuple[None, None, None]]
 WSGIEnvironment: TypeAlias = MutableMapping[str, Any]
 
 
 class StartResponse(Protocol):
-    def __call__(self, __status: str, __headers: Headers, __exc_info: OptExcInfo | None = ..., /) -> Any: ...
+    def __call__(self, __status: str, __headers: Headers, __exc_info: OptExcInfo | None = ...) -> Any: ...
 
 
 class InputStream(Iterable[bytes], Protocol):
-    def read(self, __size: int = ..., /) -> bytes: ...
-    def readline(self, __size: int = ..., /) -> bytes: ...
-    def readlines(self, __hint: int = ..., /) -> Iterator[bytes]: ...
-
-
-class SupportsKeysAndGetItem(Protocol[_K, _V]):
-    def __getitem__(self, __key: _K, /) -> _V: ...
-    def keys(self) -> Iterable[_K]: ...
+    def read(self, __size: int = ...) -> bytes: ...
+    def readline(self, __size: int = ...) -> bytes: ...
+    def readlines(self, __hint: int = ...) -> Iterator[bytes]: ...
